;; Generated package description from hydra.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "hydra" "0.15.0.0.20250316.223951" "Make bindings that stick around." '((emacs "24.4") (lv "0")) :commit "84a9db01eff1fcc52ad054c5ad3c22316739573f" :authors '(("Oleh Krehel" . "ohwoeowho@gmail.com")) :maintainer '("Oleh Krehel" . "ohwoeowho@gmail.com") :keywords '("bindings") :url "https://github.com/abo-abo/hydra")
